from django.conf import settings
from django.test import TestCase, Client
from django.core.urlresolvers import reverse
from django.core.management import call_command
from ohm2_handlers_light import utils as h_utils

try:
	import simplejson as json
except Exception:
	import json


							 	
class ApiTestCase(TestCase):
	
	test_username = "testusername"
	test_email = "slipktonesraton@gmail.com"
	test_password = h_utils.random_string(10)
	test_alias = "testalias"

	test_long_email = h_utils.random_string(30) + "@fakedomain.com"
	test_long_email_password = h_utils.random_string(10)

	test_email_2 = "oliverherreram@gmail.com"
		
	def setUp(self):
		h_settings.PRINT_BASE_ERRORS = False

		call_command('countries_init')
		call_command('currencies_init')

		user = accounts_utils.create_user(self.test_username, self.test_email, self.test_password)		
		

	def test_signup_success(self):
		SIGNUPS_ENABLED_ORIGINAL = settings.SIGNUPS_ENABLED
		settings.SIGNUPS_ENABLED = True	

		c = Client()
		url = reverse("accounts:api_signup", kwargs = {"version" : "1"})
		
		data = {
			"username" : h_utils.random_string(10),
			"password" : h_utils.random_string(10),
			"email" : "asd@asd.com",
		}
		response = c.post(url, data = json.dumps(data), content_type = 'application/json')
		self.assertEqual(response.status_code, 200)
		res = json.loads(response.content)
		error = True if res["error"] else False
		if error:
			print(res["error"])

		self.assertEqual(error, False)	
		settings.SIGNUPS_ENABLED = SIGNUPS_ENABLED_ORIGINAL
		
			


	def test_login_by_username_success(self):
		ENABLE_ALIAS_LOGIN_ORIGINAL = settings.ENABLE_ALIAS_LOGIN
		settings.ENABLE_ALIAS_LOGIN = False

		c = Client()
		url = reverse("accounts:api_login", kwargs = {"version" : "1"})
		
		data = {
			"username" : self.test_username,
			"password" : self.test_password,
		}
		response = c.post(url, data = json.dumps(data), content_type = 'application/json')
		self.assertEqual(response.status_code, 200)
		res = json.loads(response.content)
		error = True if res["error"] else False
		if error:
			print(res["error"])

		self.assertEqual(error, not res["ret"])	


		settings.ENABLE_ALIAS_LOGIN = ENABLE_ALIAS_LOGIN_ORIGINAL



	
	def test_logout_by_username_success(self):
		ENABLE_ALIAS_LOGIN_ORIGINAL = settings.ENABLE_ALIAS_LOGIN
		settings.ENABLE_ALIAS_LOGIN = False

		c = Client()
		url = reverse("accounts:api_login", kwargs = {"version" : "1"})
		
		data = {
			"username" : self.test_username,
			"password" : self.test_password,
		}
		response = c.post(url, data = json.dumps(data), content_type = 'application/json')
		self.assertEqual(response.status_code, 200)
		res = json.loads(response.content)
		error = True if res["error"] else False
		if error:
			print(res["error"])

		self.assertEqual(error, not res["ret"])	






		url = reverse("accounts:api_logout", kwargs = {"version" : "1"})
		
		data = {
		}
		response = c.post(url, data = json.dumps(data), content_type = 'application/json')
		self.assertEqual(response.status_code, 200)
		res = json.loads(response.content)
		error = True if res["error"] else False
		if error:
			print(res["error"])

		self.assertEqual(error, not res["ret"])

		settings.ENABLE_ALIAS_LOGIN = ENABLE_ALIAS_LOGIN_ORIGINAL	


from django.shortcuts import render, redirect
from django.core.urlresolvers import reverse
from django.http import HttpResponse, JsonResponse
from django.contrib.auth.decorators import login_required
from django.views.decorators.csrf import csrf_exempt
from rest_framework.authentication import SessionAuthentication, TokenAuthentication
from rest_framework.permissions import AllowAny, IsAuthenticated
from rest_framework.decorators import authentication_classes, permission_classes, api_view
from ohm2_handlers_light.parsers import get_as_or_get_default
from . import dispatcher



@api_view(['POST'])
@permission_classes((AllowAny,))
def signup(request):
	keys = (
		("username", "username", ""),
		("email", "email", ""),
		("password", "password", ""),
	)
	ret, error = dispatcher.signup(request, get_as_or_get_default(request.data, keys))
	if error:
		return JsonResponse({"error" : "an error occured"})
	return JsonResponse(ret)


@api_view(['POST'])
@permission_classes((AllowAny,))
def login(request):
	keys = (
		("username", "username", ""),
		("password", "password", ""),
	)
	ret, error = dispatcher.login(request, get_as_or_get_default(request.data, keys))
	if error:
		return JsonResponse({"error" : "an error occured"})
	return JsonResponse(ret)	


from django.utils.translation import ugettext as _

BASE_ERROR_CODE = 499264

USER_ALREADY_LOGGED_IN = {
	"code" : BASE_ERROR_CODE | 1,
	"message" : _("The user is already logged in"),
}
USER_ALREADY_EXIST = {
	"code" : BASE_ERROR_CODE | 2,
	"message" : _("The user already exist"),
}
THE_PASSWORD_IS_NOT_SECURE = {
	"code" : BASE_ERROR_CODE | 3,
	"message" : _("The password is not secure"),
}
INVALID_EMAIL = {
	"code" : BASE_ERROR_CODE | 4,
	"message" : _("The email is not valid"),
}
SIGNUPS_DISABLED = {
	"code" : BASE_ERROR_CODE | 5,
	"message" : _("Signups are currently disabled"),
}


from ohm2_accounts_light.decorators import ohm2_accounts_light_safe_request
from ohm2_accounts_light import utils as ohm2_accounts_light_utils
from ohm2_accounts_light import settings
from . import errors




@ohm2_accounts_light_safe_request
def signup(request, params):
	p = h_utils.cleaned(params,
						(
							("string", "username", 1),
							("string", "password", 1),
							("string", "email", 0),
						))


	
	if request.user.is_authenticated():
		return {"error": errors.USER_ALREADY_LOGGED_IN}
	
	elif ohm2_accounts_light_utils.user_exist(username = p["username"]):
		return {"error" : errors.USER_ALREADY_EXIST}

	elif settings.CHECK_PASSWORD_SECURE and not ohm2_accounts_light_utils.is_password_secure(p["password"]):
		return {"error" : errors.THE_PASSWORD_IS_NOT_SECURE}

	elif len(p["email"]) > 0 and not h_utils.is_email_valid(p["email"]):
		return {"error" : errors.INVALID_EMAIL}

	elif not settings.SIGNUPS_ENABLED:
		return {"error" : errors.SIGNUPS_DISABLED}

	else:
		pass


	username, password, email = p["username"], p["password"], p["email"]

	if len(email) == 0 and h_utils.is_email_valid(username):
		email = username

	
	user = ohm2_accounts_light_utils.create_user(username, email, password)

	ret = {
		"error" : None,
		"ret" : True,
	}
	return ret



@ohm2_accounts_light_safe_request
def login(request, params):
	p = h_utils.cleaned(params,
						(
							("string", "username", 1),
							("string", "password", 1),
							("string", "email", 0),
						))


	
	

	ret = {
		"error" : None,
		"ret" : True,
	}
	return ret



from django.core.exceptions import ObjectDoesNotExist, ValidationError
from django.contrib.auth.models import User
from django.contrib.auth import authenticate, login, logout
from django.contrib.auth.password_validation import validate_password, password_changed, password_validators_help_texts
from django.db.models import Q
from ohm2_handlers_light import utils as h_utils

def create_user(username, email, password, **kwargs):

	user = User.objects.create_user(username = username,
								    email = email,
								    password = password)
	

	return user

def get_user(**kwargs):
	return h_utils.db_get(User, **kwargs)

def get_or_none_user(**kwargs):
	return h_utils.db_get_or_none(User, **kwargs)

def filter_user(**kwargs):
	return h_utils.db_filter(obj = User, **kwargs)

def user_exist(**kwargs):
	if filter_user(**kwargs).count() > 0:
		return True
	return False

def change_password(user, password):
	user.set_password(password)
	user.save()
	return user

def user_authenticate(username, password):
	return authenticate(username = username, password = password)

def user_login(request, auth_user):
	return login(request, auth_user)

def user_logout(request):
	return logout(request)


def validate_current_password(password, user = None, password_validators = None):
	try:
		validate_password(password, user, password_validators)
	except ValidationError as e:
		return [reason for reason in e]
	return []

def is_password_secure(password, user = None, password_validators = None):
	errors = validate_current_password(password, user, password_validators)
	if len(errors) == 0:
		return True
	return False